@@ -76,19 +76,19 @@ class email_sender :
         return self.executor(recipients, message)
 
 class lazy_LINE :
-    def __init__(self, token) :
+    def __init__(self, token, timeout = 60) :
         self.token = token
+        self.timeout = timeout
     
     def help(self) :
         helper = 'Main Page : https://notify-bot.line.me/en\n'
         helper += 'Doc : https://notify-bot.line.me/doc/en\n'
         helper += 'Sticker List : https://developers.line.biz/en/docs/messaging-api/sticker-list'
         print(helper)
-    
-    def send(self, message , stickerPackageId = '', stickerId = '', notification = True, picture = '') :
-        payload = {}
-        payload['message'] = message
-        payload['notificationDisabled'] = not notification
+
+    def send(self, message , stickerPackageId = '', stickerId = '', notification = True, picture = '' , timeout = None) :
+        if timeout == None : timeout = self.timeout
+        payload = {'message' : message , 'notificationDisabled' : not notification}
 
         if (stickerPackageId != '') & (stickerId != '') :
             payload['stickerPackageId'] = stickerPackageId
@@ -97,11 +97,12 @@ class lazy_LINE :
         if picture == '' : 
             r = requests.post('https://notify-api.line.me/api/notify'
                                 , headers={'Authorization' : 'Bearer {}'.format(self.token)}
-                                , params = payload)
+                                , params = payload, timeout = timeout)
         else :
             r = requests.post('https://notify-api.line.me/api/notify'
                                 , headers={'Authorization' : 'Bearer {}'.format(self.token)}
-                                , params = payload , files = {'imageFile': open(picture, 'rb')})
+                                , params = payload , timeout = timeout
+                                , files = {'imageFile': open(picture, 'rb')})
         return r
 
 def log_csv(file_name,msg_in):
@@ -148,4 +149,48 @@ def healthcheck(cpu_interval = 1):
              'hostip' : [pc_ip],
              'cpu_use': [cpu_use], 
              'ram_use': [ram_use],
-             'rom_use': [rom_use]}
\ No newline at end of file
+             'rom_use': [rom_use]}
+
+
+
+def timeout(timeout):
+    """
+    Refer to https://stackoverflow.com/questions/21827874/timeout-a-function-windows
+    
+    Usage
+    
+    func = timeout(timeout=XXX)(My_Function)
+    try: func()
+    except: pass
+
+    or
+
+    @timeout(XX)
+    def My_Function :
+        ....
+    """
+    from threading import Thread
+    import functools
+    def deco(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            res = [Exception('function [%s] timeout [%s seconds] exceeded!' % (func.__name__, timeout))]
+            def newFunc():
+                try:
+                    res[0] = func(*args, **kwargs)
+                except Exception as e:
+                    res[0] = e
+            t = Thread(target=newFunc)
+            t.daemon = True
+            try:
+                t.start()
+                t.join(timeout)
+            except Exception as je:
+                print ('error starting thread')
+                raise je
+            ret = res[0]
+            if isinstance(ret, BaseException):
+                raise ret
+            return ret
+        return wrapper
+    return deco
\ No newline at end of file
